'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var path = require('path');
var promises = require('fs/promises');
var core = require('@prpl/core');
var hljs = require('highlight.js/lib/core');
var htmlEscaper = require('html-escaper');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      }
    });
  }
  n['default'] = e;
  return Object.freeze(n);
}

var hljs__default = /*#__PURE__*/_interopDefaultLegacy(hljs);

var PRPLPluginCodeHighlightExtension;
(function (PRPLPluginCodeHighlightExtension) {
    PRPLPluginCodeHighlightExtension["html"] = ".html";
})(PRPLPluginCodeHighlightExtension || (PRPLPluginCodeHighlightExtension = {}));
var PRPLPluginCodeHighlightCachePartitionKey;
(function (PRPLPluginCodeHighlightCachePartitionKey) {
    PRPLPluginCodeHighlightCachePartitionKey["codeHighlight"] = "plugin-code-highlight";
})(PRPLPluginCodeHighlightCachePartitionKey || (PRPLPluginCodeHighlightCachePartitionKey = {}));
async function highlightCode(args) {
    const { cachePartitionKey } = args || {};
    if (!cachePartitionKey) {
        await core.PRPLCache?.define(PRPLPluginCodeHighlightCachePartitionKey.codeHighlight);
    }
    const partitionKey = cachePartitionKey || PRPLPluginCodeHighlightCachePartitionKey.codeHighlight;
    const distTree = await core.generateOrRetrieveFileSystemTree({
        entityPath: path.resolve('dist'),
        partitionKey,
        readFileRegExp: new RegExp(PRPLPluginCodeHighlightExtension.html)
    });
    async function walkDistTree(items) {
        for (let i = 0; i < items?.length; i++) {
            switch (items?.[i]?.entity) {
                case 'file':
                    try {
                        if (items?.[i]?.extension !== PRPLPluginCodeHighlightExtension.html) {
                            break;
                        }
                        const codeBlocks = [
                            ...items?.[i]?.src?.matchAll(/<code class="language-(.*?)">(.*?)<\/code>/gs)
                        ];
                        if (!codeBlocks?.length) {
                            break;
                        }
                        const mappedAliases = {
                            html: 'xml'
                        };
                        const escapedLanguages = {
                            xml: true
                        };
                        for (let [block, language, code] of codeBlocks) {
                            language = mappedAliases?.[language] ?? language;
                            const languagePath = `highlight.js/lib/languages/${language}`;
                            const languageRegistered = Boolean(hljs__default['default']?.getLanguage(language));
                            if (!languageRegistered) {
                                const { default: languageGrammar } = await Promise.resolve().then(function () { return /*#__PURE__*/_interopNamespace(require(languagePath)); });
                                if (typeof languageGrammar !== 'function') {
                                    core.log.error(`Failed to load language grammar for ${language}`);
                                    return;
                                }
                                hljs__default['default']?.registerLanguage(language, languageGrammar);
                            }
                            code = escapedLanguages?.[code] ? htmlEscaper.escape(code) : code;
                            const highlightedCode = hljs__default['default']?.highlight(code, { language })?.value;
                            const reconstructedBlock = `<code class="language-${language}">${highlightedCode}</code>`;
                            items[i].src = items?.[i]?.src?.replace(block, reconstructedBlock);
                        }
                        await promises.writeFile(items?.[i]?.targetFilePath, items?.[i]?.src);
                    }
                    catch (error) {
                        core.log.error(`Failed to highlight code block in file '${items?.[i]?.srcRelativeFilePath}'. Error:`, error?.message);
                    }
                    break;
                case 'directory':
                    await walkDistTree(items?.[i]?.children);
                    break;
            }
        }
    }
    await walkDistTree(distTree?.children || []);
    core.log.info('Highlighted code');
    return core.PRPLCache?.cache;
}

exports.highlightCode = highlightCode;
